@@ -1,20 +1,19 @@
 package Devel::Examine::Subs;
-use 5.012;
+use 5.008;
 use warnings;
 use strict;
 
-our $VERSION = '1.44_02';
+our $VERSION = '1.57_01';
 
 use Carp;
 use Data::Compare;
 use Data::Dumper;
 use Devel::Examine::Subs::Engine;
 use Devel::Examine::Subs::Preprocessor;
-use Devel::Examine::Subs::Prefilter;
+use Devel::Examine::Subs::Postprocessor;
 use File::Basename;
 use File::Copy;
 use File::Edit::Portable;
-use File::Find;
 use PPI;
 use Symbol qw(delete_package);
 
@@ -28,7 +27,7 @@ BEGIN {
         import Devel::Trace::Subs qw(trace);
     };
 
-    if ($@){
+    if ($@ && ! exists $Devel::Examine::Subs{'trace'}){
         *trace = sub {};
     }
 };
@@ -41,7 +40,8 @@ sub new {
    
     # set up for tracing
 
-    if ($ENV{DTS_ENABLE} && $ENV{DES_TRACE}){
+    if ($ENV{DES_TRACE}){
+        $ENV{DTS_ENABLE} = 1;
         $ENV{TRACE} = 1;
     }
 
@@ -53,38 +53,23 @@ sub new {
 
     # default configs
 
-    $self->{namespace} = 'Devel::Examine::Subs';
+    $self->{namespace} = __PACKAGE__;
     $self->{params}{regex} = 1;
 
     $self->_config($p);
 
     return $self;
 }
-sub run {
-
+sub all {
+    
     trace() if $ENV{TRACE};
 
     my $self = shift;
-    my $p = shift;
+    my $p = $self->_params(@_);
 
-    $self->_config($p);
-
-    $self->_run_end(0);
-
-    my $struct;
-
-    if ($self->{params}{directory}){
-        $struct = $self->_run_directory;
-    }
-    else {
-        $struct = $self->_core;
-        $self->_write_file if $self->{write_file_contents};
-    }
-
-    $self->_run_end(1);
-    $self->_clean_core_config;
-
-    return $struct;
+    $self->{params}{engine} = 'all';
+    
+    $self->run($p);
 }
 sub has {
     
@@ -93,7 +78,7 @@ sub has {
     my $self = shift;
     my $p = $self->_params(@_);
 
-    $self->{params}{pre_filter} = 'file_lines_contain';
+    $self->{params}{post_proc} = 'file_lines_contain';
     $self->{params}{engine} = 'has';
     
     $self->run($p);
@@ -109,17 +94,6 @@ sub missing {
     
     $self->run($p);
 }
-sub all {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    $self->{params}{engine} = 'all';
-    
-    $self->run($p);
-}
 sub lines {
     
     trace() if $ENV{TRACE};
@@ -130,7 +104,7 @@ sub lines {
     $self->{params}{engine} = 'lines';
     
     if ($self->{params}{search} || $p->{search}){
-        $self->{params}{pre_filter} = 'file_lines_contain';
+        $self->{params}{post_proc} = 'file_lines_contain';
     }
 
     $self->run($p);
@@ -171,7 +145,7 @@ sub objects {
     my $self = shift;
     my $p = $self->_params(@_);
 
-    $self->{params}{pre_filter} = 'subs';
+    $self->{params}{post_proc} = 'subs';
     $self->{params}{engine} = 'objects';
 
     $self->run($p);
@@ -183,46 +157,13 @@ sub search_replace {
     my $self = shift;
     my $p = $self->_params(@_);
 
-    $self->{params}{pre_filter}
+    $self->{params}{post_proc}
       = ['file_lines_contain', 'subs', 'objects'];
 
     $self->{params}{engine} = 'search_replace';
 
     $self->run($p);
 }
-sub inject_after {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    if (! $p->{injects} && ! $self->{params}{injects}){
-        $p->{injects} = 1;
-    }
-
-    $self->{params}{pre_filter}
-      = ['file_lines_contain', 'subs', 'objects'];
-
-    $self->{params}{engine} = 'inject_after';
-
-    $self->run($p);
-}
-sub inject {
-    
-    trace() if $ENV{TRACE};
-    my $self = shift;
-    my $p = $self->_params(@_);
-
-    # inject_use/inject_after_sub_def are preprocs
-
-    if ($p->{inject_use} || $p->{inject_after_sub_def}){
-        $self->{params}{pre_proc} = 'inject';
-        $self->{params}{pre_proc_return} = 1;
-    }
-
-    $self->run($p);
-}
 sub replace {
 
     trace() if $ENV{TRACE};
@@ -235,13 +176,44 @@ sub replace {
 
     $self->run($p);
 }
-sub remove {
+sub inject_after {
     
     trace() if $ENV{TRACE};
 
     my $self = shift;
     my $p = $self->_params(@_);
 
+    if (! $p->{injects} && ! $self->{params}{injects}){
+        $p->{injects} = 1;
+    }
+
+    $self->{params}{post_proc}
+      = ['file_lines_contain', 'subs', 'objects'];
+
+    $self->{params}{engine} = 'inject_after';
+
+    $self->run($p);
+}
+sub inject {
+    trace() if $ENV{TRACE};
+    my $self = shift;
+    my $p = $self->_params(@_);
+
+    # inject_use/inject_after_sub_def are preprocs
+
+    if ($p->{inject_use} || $p->{inject_after_sub_def} || defined $p->{line_num}){
+        $self->{params}{pre_proc} = 'inject';
+        $self->{params}{pre_proc_return} = 1;
+    }
+
+    $self->run($p);
+}
+sub remove {
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $p = $self->_params(@_);
+
     $self->{params}{pre_proc} = 'remove';
     $self->{params}{pre_proc_return} = 1;
 
@@ -249,7 +221,7 @@ sub remove {
 }
 
 #
-# publicly available semi-private methods
+# publicly available semi-private developer methods
 #
 
 sub add_functionality {
@@ -267,22 +239,43 @@ sub add_functionality {
 
     my @allowed = qw(
                     pre_proc
-                    pre_filter
+                    post_proc
                     engine
     );
 
+    my $is_allowed = 0;
 
-    if (! (grep {$to_add eq $_} @allowed)){
+    for (@allowed){
+        if ($_ eq $to_add){
+            $is_allowed = 1;
+            last;
+        }
+    }
+    
+    if (! $is_allowed){
         croak "Adding a non-allowed piece of functionality...\n";
     }
 
     my %dt = (
+            pre_proc => sub {
+                        trace() if $ENV{TRACE};
+                        return $in_prod
+                        ? $INC{'Devel/Examine/Subs/Preprocessor.pm'}
+                        : 'lib/Devel/Examine/Subs/Preprocessor.pm';
+                      },
+            post_proc => sub {
+                        trace() if $ENV{TRACE};
+                        return $in_prod
+                        ? $INC{'Devel/Examine/Subs/Postprocessor.pm'}
+                        : 'lib/Devel/Examine/Subs/Postprocessor.pm';
+                      },
             engine => sub {
                         trace() if $ENV{TRACE};
                         return $in_prod
                         ? $INC{'Devel/Examine/Subs/Engine.pm'}
                         : 'lib/Devel/Examine/Subs/Engine.pm';
                       },
+
     );
 
     my $caller = (caller)[1];
@@ -306,42 +299,41 @@ sub add_functionality {
 
     my $file = $dt{$to_add}->();
 
-    my $des = Devel::Examine::Subs->new({
+    my $des = Devel::Examine::Subs->new(
                                     file => $file,
-                                    pre_filter => 'end_of_last_sub',
-                                });
+                                    engine => 'objects', 
+                                    post_proc => [qw(subs end_of_last_sub)],
+                                );
+    $p = {
+        engine => 'objects', 
+        post_proc => [qw(subs end_of_last_sub)],
+        post_proc_return => 1,
+    };
 
-    $p->{write_file_contents} = \@code;
-}
-sub pre_procs {
-    
-    trace() if $ENV{TRACE};
+    my $start_writing = $des->run($p);
 
-    my $self = shift;
-    my $module = $self->{namespace} . "::Preprocessor";
-    my $pre_proc = $module->new;
+    my $rw = File::Edit::Portable->new;
 
-    my @pre_procs;
+    $rw->splice(file => $file, insert => \@code, line => $start_writing);
 
-    for (keys %{$pre_proc->_dt}){
-        push @pre_procs, $_ if $_ !~ /^_/;
+    my $sub_name;
+
+    if ($code[0] =~ /sub\s+(\w+)\s+\{/){
+        $sub_name = $1;
     }
-    return @pre_procs;
-}
-sub pre_filters {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $module = $self->{namespace} . "::Prefilter";
-    my $pre_filter = $module->new;
-
-    my @pre_filters;
-
-    for (keys %{$pre_filter->_dt}){
-        push @pre_filters, $_ if $_ !~ /^_/;
+    else {
+        croak "add_functionality() couldn't extract the sub name.";
     }
-    return @pre_filters;
+
+    my @insert = ("        $sub_name => \\&$sub_name,");
+
+    my @ret = $rw->splice(
+        file => $file, 
+        find => 'my\s+\$dt\s+=\s+\{',
+        insert => \@insert,
+    );
+
+    return 1;
 }
 sub engines {
     
@@ -358,6 +350,61 @@ sub engines {
     }
     return @engines;
 }
+sub pre_procs {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $module = $self->{namespace} . "::Preprocessor";
+    my $pre_proc = $module->new;
+
+    my @pre_procs;
+
+    for (keys %{$pre_proc->_dt}){
+        push @pre_procs, $_ if $_ !~ /^_/;
+    }
+    return @pre_procs;
+}
+sub post_procs {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $module = $self->{namespace} . "::Postprocessor";
+    my $post_proc = $module->new;
+
+    my @post_procs;
+
+    for (keys %{$post_proc->_dt}){
+        push @post_procs, $_ if $_ !~ /^_/;
+    }
+    return @post_procs;
+}
+sub run {
+
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $p = shift;
+
+    $self->_config($p);
+
+    $self->_run_end(0);
+
+    my $struct;
+
+    if ($self->{params}{directory}){
+        $struct = $self->_run_directory;
+    }
+    else {
+        $struct = $self->_core;
+        $self->_write_file if $self->{write_file_contents};
+    }
+
+    $self->_run_end(1);
+
+    return $struct;
+}
 sub valid_params {
     
     trace() if $ENV{TRACE};
@@ -370,67 +417,6 @@ sub valid_params {
 # private methods
 #
 
-sub _run_directory {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $p = shift;
-
-    my @files;
-
-    my $dir = $self->{params}{file};
-
-    find({wanted => sub {
-            
-            trace() if $ENV{TRACE};
-
-            return if ! -f;
-
-            my @extensions = @{$self->{params}{extensions}};
-            my $exts = join('|', @extensions);
-
-            if ($_ !~ /\.(?:$exts)$/i){
-                return;
-            }
-
-            my $file = "$File::Find::name";
-
-            push @files, $file;
-          },
-            no_chdir => 1
-        }, $dir
-    );
-
-    my %struct;
-
-    for my $file (@files){
-        
-        $self->{params}{file} = $file;
-        my $data = $self->_core($p);
-
-        $self->_write_file if $self->{write_file_contents};
-
-        my $exists = 0;
-        $exists = %$data if ref($data) eq 'HASH';
-        $exists = @$data if ref($data) eq 'ARRAY';
-
-        $struct{$file} = $data if $exists;
-    }
-
-    return \%struct;
-}
-sub _run_end {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $value = shift;
-
-    $self->{run_end} = $value if defined $value;
-
-    return $self->{run_end};
-}
 sub _cache {
     
     trace() if $ENV{TRACE};
@@ -463,6 +449,7 @@ sub _cache_enabled {
     return $self->{params}{cache};
 }
 sub _cache_safe {
+    
     trace() if $ENV{TRACE};
 
     my $self = shift;
@@ -472,13 +459,53 @@ sub _cache_safe {
 
     return $self->{cache_safe};
 }
-sub _params {
+sub _clean_config {
     
     trace() if $ENV{TRACE};
 
     my $self = shift;
-    my %params = @_;
-    return \%params;
+    my $config_vars = shift; # href of valid params
+    my $p = shift;           # href of params passed in
+
+    for my $var (keys %$config_vars){
+       
+        last if ! $self->_run_end;
+
+        # skip if it's a persistent var
+
+        next if $config_vars->{$var} == 1;
+
+        delete $self->{params}{$var};
+    }
+
+    # delete non-valid params
+
+    for my $param (keys %$p){
+        if (! exists $config_vars->{$param}){
+            print "\n\nDES::_clean_config() deleting invalid param: $param\n";
+            delete $p->{$param};
+        }
+    }
+}
+sub _clean_core_config {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+
+    # delete params we collected after _clean_config()
+
+    delete $self->{params}{file_contents};
+
+    my @core_phases = qw(
+        pre_proc
+        post_proc
+        engine
+    );
+
+    for (@core_phases){
+        delete $self->{params}{$_};
+    }
 }
 sub _config {
     
@@ -493,6 +520,7 @@ sub _config {
 
         file => 1,
         extensions => 1,
+        maxdepth => 1,
         regex => 1,
         copy => 1,
         diff => 1,
@@ -502,7 +530,7 @@ sub _config {
         # persistent - core phases
 
         pre_proc => 1,
-        pre_filter => 1,
+        post_proc => 1,
         engine => 1,
 
         # transient
@@ -518,11 +546,11 @@ sub _config {
         module => 0,
         objects_in_hash => 0,
         pre_proc_dump => 0,
-        pre_filter_dump => 0,
+        post_proc_dump => 0,
         engine_dump => 0,
         core_dump => 0,
         pre_proc_return => 0,
-        pre_filter_return => 0,
+        post_proc_return => 0,
         engine_return => 0,
         config_dump => 0,
         cache_dump => 0,
@@ -532,6 +560,9 @@ sub _config {
         file_contents => 0,
         exec => 0,
         limit => 0,
+        line_num => 0,              # inject()
+        add_functionality => 0,
+        add_functionality_prod => 0,
     );
 
     $self->{valid_params} = \%valid_params;
@@ -579,59 +610,6 @@ sub _config {
         print Dumper $self->{params};
     }
 }
-sub _clean_config {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-    my $config_vars = shift; # href of valid params
-    my $p = shift; # href of params passed in
-
-    # params that get set after the call to _config() need
-    # to be deleted manually here
-
-    for my $var (keys %$config_vars){
-       
-        last if ! $self->_run_end;
-
-        # skip if it's a persistent var
-
-        next if $config_vars->{$var} == 1;
-
-        delete $self->{params}{$var};
-    }
-
-    # delete non-valid params
-
-    for my $param (keys %$p){
-        if (! exists $config_vars->{$param}){
-            #warn "\n\nDES::_clean_config() deleting invalid param: $param\n";
-            delete $p->{$param};
-        }
-    }
-}
-sub _clean_core_config {
-    
-    trace() if $ENV{TRACE};
-
-    my $self = shift;
-
-    # deletes core phase info after each run
-
-    # clean params that we collected after _clean_config()
-
-    delete $self->{params}{file_contents};
-
-    my @core_phases = qw(
-        pre_proc
-        pre_filter
-        engine
-    );
-
-    for (@core_phases){
-        delete $self->{params}{$_};
-    }
-}
 sub _file {
     
     trace() if $ENV{TRACE};
@@ -639,7 +617,7 @@ sub _file {
     my $self = shift;
     my $p = shift;
 
-    $self->{params}{file} = $p->{file} // $self->{params}{file};
+    $self->{params}{file} = defined $p->{file} ? $p->{file} : $self->{params}{file};
 
     # if a module was passed in, dig up the file
 
@@ -680,7 +658,8 @@ sub _file {
 
     if (-d $self->{params}{file}){
         $self->{params}{directory} = 1;
-        $self->{params}{extensions} = $p->{extensions} // [qw(pm pl)];
+        $self->{params}{extensions} 
+          = defined $p->{extensions} ? $p->{extensions} : [qw(*.pm *.pl)];
     }
     else {
         if (! $self->{params}{file} || ! -f $self->{params}{file}){
@@ -690,10 +669,21 @@ sub _file {
 
    return $self->{params}{file};
 }
-sub _read_file {
+sub _params {
     
     trace() if $ENV{TRACE};
 
+    my $self = shift;
+    my %params = @_;
+    return \%params;
+}
+sub _read_file {
+
+    # this sub prepares a temp copy of the original file,
+    # recseps changed to local platform for PPI
+
+    trace() if $ENV{TRACE};
+
     my $self = shift;
     my $p = shift;
 
@@ -705,16 +695,83 @@ sub _read_file {
     my $bak = "$basename.bak";
 
     copy $file, $bak
-      or croak "can't create backup copy of file $file!";
+      or croak "DES::_read_file() can't create backup copy $bak!";
 
     $self->{rw} = File::Edit::Portable->new;
 
-    my @file_contents
-      = $self->{rw}->read(file => $file);
+    my $fh = $self->{rw}->read($file);
+    my $tempfile = $self->{rw}->tempfile;
+    my $tempfile_name = $tempfile->filename;
+    my $platform_recsep = $self->{rw}->platform_recsep;
 
-    @{ $p->{file_contents} } = @file_contents;
+    $self->{rw}->write(
+        copy => $tempfile_name, 
+        contents => $fh, 
+        recsep => $platform_recsep
+    );
 
-    return @file_contents;
+    my $ppi_doc = PPI::Document->new($tempfile_name);
+    
+    @{ $p->{file_contents} } = split /\n/, $ppi_doc->content;
+
+    close $tempfile;
+
+    if (! $p->{file_contents}->[0]){
+        return 0;
+    }
+    else {
+        $self->{params}{file_contents} = $p->{file_contents};
+        return 1;
+    }
+}
+sub _run_directory {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $p = shift;
+
+    my $dir = $self->{params}{file};
+
+    $self->{rw} = File::Edit::Portable->new;
+
+    my @files = $self->{rw}->dir(
+                            dir => $dir,
+                            maxdepth => $self->{params}{maxdepth} || 0,
+                            types => $self->{params}{extensions},
+                            list => 1,
+                        );
+
+    my %struct;
+
+    for my $file (@files){
+        
+        $self->{params}{file} = $file;
+        my $data = $self->_core($p);
+
+        $self->_write_file if $self->{write_file_contents};
+
+        if (ref $data eq 'HASH' || ref $data eq 'ARRAY'){
+            $struct{$file} = $data;
+        }
+    }
+
+    return \%struct;
+}
+sub _run_end {
+    
+    trace() if $ENV{TRACE};
+
+    my $self = shift;
+    my $value = shift;
+
+    $self->{run_end} = $value if defined $value;
+
+    # we clean core_config here
+
+    $self->_clean_core_config if $value;
+
+    return $self->{run_end};
 }
 sub _write_file {
 
@@ -722,19 +779,21 @@ sub _write_file {
 
     my $self = shift;
 
-    my $file = $self->{params}{file};
     my $copy = $self->{params}{copy};
+    
+    my $file = $self->{params}{file};
     my $contents = $self->{write_file_contents};
 
     return if ! $file;
-
     $file = $copy if $copy;
 
+    my $write_response;
+
     eval {
-        $self->{rw}->write(file => $file, contents => $contents);
+        $write_response = $self->{rw}->write(file => $file, contents => $contents);
     };
 
-    if ($@){
+    if ($@ || ! $write_response){
         $@ = "\nDevel::Examine::Subs::_write_file() speaking...\n\n" .
              "File::Edit::Portable::write() returned a failure status.\n\n" .
              $@;
@@ -766,7 +825,7 @@ sub _core {
     if ($self->{params}{pre_proc}){
         my $pre_proc = $self->_pre_proc;
 
-        $data = $pre_proc->($p);
+        $data = $pre_proc->($p, $data);
 
         if ($self->{params}{pre_proc_dump}){
             print Dumper $data;
@@ -798,38 +857,33 @@ sub _core {
         $subs = $self->_cache($p->{file});
     }
     else {
-        $subs = $self->_proc;
+        $subs = $self->_proc($p);
     } 
-    
-    $subs = $subs // 0;
 
     return if ! $subs;
 
-    $self->{data} = $subs;
-
     # write to cache
 
     if ($self->_cache_enabled && ! $self->_cache($p->{file})){
         $self->_cache($p->{file}, $subs);
     }
 
-    # pre engine filter
+    # post processor
 
-    if ($self->{params}{pre_filter}){
-        for my $pre_filter ($self->_pre_filter){
-            $subs = $pre_filter->($p, $subs);
+    if ($self->{params}{post_proc}){
+        for my $post_proc ($self->_post_proc($p, $subs)){
+            $subs = $post_proc->($p, $subs);
             $self->{write_file_contents} = $p->{write_file_contents};
-            $self->{data} = $subs;
         }
     }  
 
-    if ($self->{params}{pre_filter_return}){
+    if ($self->{params}{post_proc_return}){
         return $subs;
     }
 
     # engine
 
-    my $engine = $self->_engine($p);
+    my $engine = $self->_engine($p, $subs);
 
     if ($self->{params}{engine}){
         $subs = $engine->($p, $subs);
@@ -850,8 +904,6 @@ sub _core {
         exit;
     }
 
-    $self->{data} = $subs;
-
     return $subs;
 }
 sub _pre_proc {
@@ -869,7 +921,7 @@ sub _pre_proc {
     }
    
     # tell _core() to return directly from the pre_processor 
-    # if necessary, and bypass pre_filter and engine
+    # if necessary, and bypass post_proc and engine
 
     if ($pre_proc eq 'module'){
        $self->{params}{pre_proc_return} = 1;
@@ -911,42 +963,42 @@ sub _proc {
     # this method is the core data collection/manipulation
     # routine (aka the 'Processor phase') for all of DES
 
-    # do not modify this sub
+    # make sure all unit tests are successful after any change
+    # to this subroutine!
 
     # if you want the data structure to look differently before 
     # reaching here, use a pre_proc. If you want it different 
-    # afterwards, use a pre_filter or an engine
+    # afterwards, use a post_proc or an engine
 
     trace() if $ENV{TRACE};
     
     my $self = shift;
-    
+    my $p = shift;
+   
     my $file = $self->{params}{file};
 
     return {} if ! $file;
 
-    my $PPI_doc = PPI::Document->new($file);
-    my $PPIsubs = $PPI_doc->find("PPI::Statement::Sub");
+    my $PPI_doc = PPI::Document->new($file);   
+    my $PPI_subs = $PPI_doc->find('PPI::Statement::Sub');
 
-    return if ! $PPIsubs;
-
-    my @file_contents = @{ $self->{params}{file_contents} };
+    return if ! $PPI_subs;
 
     my %subs;
     $subs{$file} = {};
     
-    @{$subs{$file}{contents}} = @file_contents;
+    for my $PPI_sub (@{$PPI_subs}){
 
-    for my $PPI_sub (@{$PPIsubs}){
-
-        my $include = $self->{params}{include} // [];
-        my $exclude = $self->{params}{exclude} // [];
+        my $include 
+          = defined $self->{params}{include} ? $self->{params}{include} : [];
+        my $exclude
+          = defined $self->{params}{exclude} ? $self->{params}{exclude} : [];
 
         delete $self->{params}{include} if $exclude->[0];
 
         my $name = $PPI_sub->name;
 
-        # skip over excluded subs
+        # skip over excluded (or not included) subs
 
         next if grep {$name eq $_ } @$exclude;
 
@@ -970,23 +1022,12 @@ sub _proc {
 
         $subs{$file}{subs}{$name}{num_lines} = $sub_line_count;
 
-        my $line_num = $subs{$file}{subs}{$name}{start};
-       
-        # pull out just the subroutine from the file 
-        # array and attach it to the structure
-
-        my @sub_definition = @file_contents[
-                                    $subs{$file}{subs}{$name}{start}
-                                    ..
-                                    $subs{$file}{subs}{$name}{end}
-                                   ];
-
-          $subs{$file}{subs}{$name}{contents} = \@sub_definition;
+        @{ $subs{$file}{subs}{$name}{code} } = split /\n/, $PPI_sub->content;
     }
    
     return \%subs;
 }
-sub _pre_filter {
+sub _post_proc {
     
     trace() if $ENV{TRACE};
 
@@ -994,48 +1035,48 @@ sub _pre_filter {
     my $p = shift;
     my $struct = shift;
 
-    my $pre_filter = $self->{params}{pre_filter};
+    my $post_proc = $self->{params}{post_proc};
 
-    my $pre_filter_dump = $self->{params}{pre_filter_dump};
+    my $post_proc_dump = $self->{params}{post_proc_dump};
 
-    my @pre_filters;
+    my @post_procs;
 
-    if ($pre_filter){
+    if ($post_proc){
 
-        my @pre_filter_list;
+        my @post_proc_list;
 
-        if (ref $pre_filter ne 'ARRAY'){
-            push @pre_filter_list, $pre_filter;
+        if (ref $post_proc ne 'ARRAY'){
+            push @post_proc_list, $post_proc;
         }
         else {
-            @pre_filter_list = @{$pre_filter};
+            @post_proc_list = @{$post_proc};
         }
 
-        for my $pf (@pre_filter_list){
+        for my $pf (@post_proc_list){
 
             my $cref;
 
             if (ref $pf ne 'CODE'){
 
-                my $pre_filter_module = $self->{namespace} . "::Prefilter";
-                my $compiler = $pre_filter_module->new;
+                my $post_proc_module = $self->{namespace} . "::Postprocessor";
+                my $compiler = $post_proc_module->new;
 
-                # pre_filter isn't in the dispatch table
+                # post_proc isn't in the dispatch table
 
                 if (! $compiler->exists($pf)){
-                    croak "\nDevel::Examine::Subs::_pre_filter() " .
-                          "speaking...\n\npre_filter '$pf' is not " .
-                          "implemented. '$pre_filter' was sent in.\n";
+                    croak "\nDevel::Examine::Subs::_post_proc() " .
+                          "speaking...\n\npost_proc '$pf' is not " .
+                          "implemented. '$post_proc' was sent in.\n";
                 }
                 
                 eval {
-                    $cref = $compiler->{pre_filters}{$pf}->();
+                    $cref = $compiler->{post_procs}{$pf}->();
                 };
         
                 if ($@){
                     $@ = "\n[Devel::Examine::Subs speaking] " .
                           "dispatch table in " .
-                          "Devel::Examine::Subs::Prefilter has a mistyped " .
+                          "Devel::Examine::Subs::Postprocessor has a mistyped " .
                           "function as a value, but the key is ok\n\n"
                     . $@;
                     croak $@;
@@ -1045,24 +1086,23 @@ sub _pre_filter {
                 $cref = $pf;
             }
 
-            if ($pre_filter_dump && $pre_filter_dump > 1){
-                $self->{params}{pre_filter_dump}--;
-                $pre_filter_dump = $self->{params}{pre_filter_dump};
+            if ($post_proc_dump && $post_proc_dump > 1){
+                $self->{params}{post_proc_dump}--;
+                $post_proc_dump = $self->{params}{post_proc_dump};
             }
 
-            if ($pre_filter_dump && $pre_filter_dump == 1){
-                my $subs = $cref->($p, $self->{data});
+            if ($post_proc_dump && $post_proc_dump == 1){
+                my $subs = $cref->($p, $struct);
                 print Dumper $subs;
                 exit;
             }
-    
-            push @pre_filters, $cref;
+            push @post_procs, $cref;
         }
     }
     else {
         return;
     }
-    return @pre_filters;
+    return @post_procs;
 }
 sub _engine {
     
@@ -1072,7 +1112,8 @@ sub _engine {
     my $p = shift;
     my $struct = shift;
 
-    my $engine = $p->{engine} // $self->{params}{engine};
+    my $engine 
+      = defined $p->{engine} ? $p->{engine} : $self->{params}{engine};
 
     if (not $engine or $engine eq ''){
         return $struct;
@@ -1113,7 +1154,7 @@ sub _engine {
     }
 
     if ($self->{params}{engine_dump}){
-        my $subs = $cref->($p, $self->{data});
+        my $subs = $cref->($p, $struct);
         print Dumper $subs;
         exit;
     }
@@ -1155,8 +1196,8 @@ Get all the subs as objects
 
     for my $sub (@$subs){
         $sub->name;       # name of sub
-        $sub->start;      # number of first line in sub
-        $sub->end;        # number of last line in sub
+        $sub->start;      # first line number of sub in file
+        $sub->end;        # last line number of sub in file
         $sub->line_count; # number of lines in sub
         $sub->code;       # entire sub code from file
         $sub->lines;      # lines that match search term
@@ -1243,7 +1284,7 @@ Print all subs within each Perl file under a directory
 
 Most methods can include or exclude specific subs
 
-    my $has = $des->has( include => [qw(dump private)] );
+    my $has = $des->has( include => ['dump', 'private'] );
 
     my $missing = $des->missing( exclude => ['this', 'that'] );
 
@@ -1272,15 +1313,15 @@ Mandatory parameters: C<file =E<gt> $filename>
 
 Instantiates a new object. If C<$filename> is a directory, we'll iterate
 through it finding all Perl files. If C<$filename> is a module name
-(eg: C<Data::Dumper>), we'll attempt to load the module, extract the file for
+(eg: C<Data::Dump>), we'll attempt to load the module, extract the file for
 the module, and load the file. CAUTION: this will be a production C<%INC> file
 so be careful.
 
 Only specific params are guaranteed to stay persistent throughout a run on the
 same object, and are best set in C<new()>. These parameters are C<file>,
-C<extensions>, C<cache>, C<regex>, C<copy>, C<no_indent> and C<diff>.
+C<extensions>, C<maxdepth>, C<cache>, C<regex>, C<copy>, C<no_indent> and C<diff>.
 
-Note: omit this parameter if all you'll be using is the C<module()> method.
+Note: omit the C<file> parameter if all you'll be using is the C<module()> method.
 
 
 
@@ -1314,6 +1355,19 @@ The exact opposite of has.
 
 
 
+=head2 C<lines>
+
+Mandatory parameters: C<search =E<gt> 'text'>
+
+Gathers together all line text and line number of all subs where the
+subroutine contains lines matching the search term.
+
+Returns a hash reference with the subroutine name as the key, the value being
+an array reference which contains a hash reference in the format line_number
+=E<gt> line_text.
+
+
+
 
 =head2 C<objects>
 
@@ -1331,7 +1385,7 @@ See L<SYNOPSIS> for the structure of each object.
 
 
 
-=head2 C<module('Module::Name'>
+=head2 C<module('Module::Name')>
 
 Mandatory parameters: C<'Module::Name'>. Note that this is one public method
 that takes its parameter in string format (as opposed to hash format).
@@ -1342,17 +1396,6 @@ module's namespace symbol table.
 
 
 
-=head2 C<lines>
-
-Mandatory parameters: C<search =E<gt> 'text'>
-
-Gathers together all line text and line number of all subs where the
-subroutine contains lines matching the search term.
-
-Returns a hash reference with the subroutine name as the key, the value being
-an array reference which contains a hash reference in the format line_number
-=E<gt> line_text.
-
 
 
 
@@ -1371,35 +1414,6 @@ with '.bak', but don't confuse this feature with the 'copy' parameter.
 
 
 
-
-=head2 C<inject>
-
-Parameters: C<inject_use =E<gt> ['use Module::Name', 'use Module2::Name']> or
-C<inject_after_sub_def =E<gt> ['code line 1;', 'code line 2;']>
-
-C<inject_use> will inject the statements prior to all existing C<use>
-statements that already exist in the file(s). If none are found, will inject
-the statements right after a C<Package> statement if found.
-
-Technically, you don't have to inject a C<use> statement, but I'd advise it.
-
-C<inject_after_sub_def> will inject the lines of code within the array
-reference value immediately following all sub definitions in a file.
-Next line indenting is used, and sub definitions with their opening brace
-on a separate line than the definition itself is caught.
-
-
-
-=head2 C<remove>
-
-Parameters: C<delete =E<gt> ['string1', 'string2']>
-
-Deletes from the file(s) the entire lines that contain the search terms.
-
-This method is file based... the work happens prior to digging up subs, hence
-C<exclude>, C<include> and other sub-based parameters have no effect.
-
-
 =head2 C<replace>
 
 Parameters: C<exec =E<gt> $cref, limit =E<gt> 1>
@@ -1415,6 +1429,14 @@ Returns the number of lines changed in file mode, and an empty hashref in direct
 
 
 
+
+
+
+
+
+
+
+
 =head2 C<inject_after>
 
 Mandatory parameters: C<search =E<gt> 'this', code =E<gt> \@code>
@@ -1455,10 +1477,42 @@ details.
 
 
 
+=head2 C<inject>
+
+Parameters (all are mutually exclusive, use only one):
+
+C<line_num =E<gt> 33> with C<code =E<gt> \@code> or,
+C<inject_use =E<gt> ['use Module::Name', 'use Module2::Name']> or,
+C<inject_after_sub_def =E<gt> ['code line 1;', 'code line 2;']>
+
+C<line_num> will inject the block of code in the array reference immediately after the line number specified.
+
+C<inject_use> will inject the statements prior to all existing C<use>
+statements that already exist in the file(s). If none are found, will inject
+the statements right after a C<Package> statement if found.
+
+Technically, you don't have to inject a C<use> statement, but I'd advise it.
+
+C<inject_after_sub_def> will inject the lines of code within the array
+reference value immediately following all sub definitions in a file.
+Next line indenting is used, and sub definitions with their opening brace
+on a separate line than the definition itself is caught.
 
 
 
 
+=head2 C<remove>
+
+Parameters: C<delete =E<gt> ['string1', 'string2']>
+
+Deletes from the file(s) the entire lines that contain the search terms.
+
+This method is file based... the work happens prior to digging up subs, hence
+C<exclude>, C<include> and other sub-based parameters have no effect.
+
+
+
+=head1 C<DEVELOPER METHODS>
 
 =head2 C<valid_params>
 
@@ -1480,7 +1534,7 @@ C<has()> public method:
 
     my $params = {
             search => 'text',
-            pre_filter => 'file_lines_contain',
+            post_proc => 'file_lines_contain',
             engine => 'has',
     };
 
@@ -1495,8 +1549,7 @@ it easy to write new engines and for testing.
 
 =head2 C<add_functionality>
 
-WARNING!: This method is highly experimental and is used for developing
-internal processors only. Only 'engine' is functional, and only half way.
+WARNING!: This method is for development of this distribution only!
 
 While writing new processors, set the processor type to a callback within the
 local working file. When the code performs the actions you want it to, put a
@@ -1506,8 +1559,7 @@ specified processor, and configure it for use. See
 C<examples/write_new_engine.pl> for an example of creating a new 'engine'
 processor.
 
-
-
+Returns 1 on success.
 
 Parameters:
 
@@ -1516,7 +1568,7 @@ Parameters:
 =item C<add_functionality>
 
 Informs the system which type of processor to inject and configure. Permitted
-values are 'pre_proc', 'pre_filter' and 'engine'.
+values are 'pre_proc', 'post_proc' and 'engine'.
 
 =item C<add_functionality_prod>
 
@@ -1541,7 +1593,7 @@ change the copy. Useful for verifying the changes took properly.
 =back
 
 
-=head2 C<pre_procs>, C<pre_filters>, C<engines>
+=head2 C<pre_procs>, C<post_procs>, C<engines>
 
 For development. Returns the list of the respective built-in callbacks.
 
@@ -1579,15 +1631,20 @@ persistent until changed again.
 
 State: Persistent
 
-Default: C<[qw(pm pl)]>
+Default: C<['*.pm', '*.pl')]>
 
 By default, we load only C<*.pm> and C<*.pl> files. Use this parameter to load
 different files. Only useful when a directory is passed in as opposed to a
 file. This parameter is persistent until manually reset and should be set in
 C<new()>.
 
-Values: Array reference where each element is the name of the extension
-(less the dot). For example, C<[qw(pm pl)]> is the default.
+Values: Array reference where each element is the names of files to find. Any wildcard or regex that are valid in L<File::Find::Rule's|http://search.cpan.org/~rclamp/File-Find-Rule-0.33/lib/File/Find/Rule.pm> are valid here. For example, C<[qw(*.pm *.pl)]> is the default.
+
+
+=item C<maxdepth>
+
+When running in directory mode, how many levels deep do you want to traverse? Default is unlimited. Set to a positive integer to set.
+
 
 =item C<cache>
 
@@ -1698,7 +1755,7 @@ integer to inject after all.
 
 
 
-=item C<pre_proc_dump>, C<pre_filter_dump>, C<engine_dump>, C<cache_dump>,
+=item C<pre_proc_dump>, C<post_proc_dump>, C<engine_dump>, C<cache_dump>,
 C<core_dump>
 
 State: Transient
@@ -1711,15 +1768,15 @@ Print to STDOUT using Data::Dumper the structure of the data following the
 respective phase. The C<core_dump> will print the state of the data, as well
 as the current state of the entire DES object.
 
-NOTE: The 'pre_filter' phase is run in such a way that pre-filters can be
-daisy-chained. Due to this reason, the value of C<pre_filter_dump> works a
+NOTE: The 'post_proc' phase is run in such a way that the filters can be
+daisy-chained. Due to this reason, the value of C<post_proc_dump> works a
 little differently. For example:
 
-    pre_filter => ['one', 'two'];
+    post_proc => ['one', 'two'];
 
 ...will execute filter 'one' first, then filter 'two' with the data that came
 out of filter 'one'. Simply set the value to the number that coincides with
-the location of the filter. For instance, C<pre_filter_dump =E<gt> 2;> will
+the location of the filter. For instance, C<post_proc_dump =E<gt> 2;> will
 dump the output from the second filter and likewise, C<1> will dump after the
 first.
 
@@ -1728,7 +1785,7 @@ will continue. Set this parameter to an integer larger than one to have the
 application C<exit> immediately after dumping the cache to STDOUT.
 
 
-=item C<pre_proc_return>, C<pre_filter_return>, C<engine_return>
+=item C<pre_proc_return>, C<post_proc_return>, C<engine_return>
 
 State: Transient
 
@@ -1738,8 +1795,8 @@ Returns the structure of data immediately after being processed by the
 respective phase. Useful for writing new 'phases'. (See "SEE ALSO" for
 details).
 
-NOTE: C<pre_filter_return> does not behave like C<pre_filter_dump>. It will
-only return after all pre-filters have executed.
+NOTE: C<post_proc_return> does not behave like C<post_proc_dump>. It will
+only return after all post_procs have executed.
 
 
 
@@ -1755,7 +1812,7 @@ configuration parameters.
 
 
 
-=item C<pre_proc, pre_filter, engine>
+=item C<pre_proc, post_proc, engine>
 
 State: Transient
 
@@ -1767,7 +1824,7 @@ used by the C<run()> command.
 C<engine> and C<pre_proc> take either a single string that contains a valid
 built-in callback, or a single code reference of a custom callback.
 
-C<pre_filter> works a lot differently. These modules can be daisy-chained.
+C<post_proc> works a lot differently. These modules can be daisy-chained.
 Like C<engine> and C<pre_proc>, you can send in a string or cref, or to chain,
 send in an aref where each element is either a string or cref. The filters
 will be executed based on their order in the array reference.
@@ -1779,15 +1836,25 @@ will be executed based on their order in the array reference.
 
 =head1 REPOSITORY
 
+=begin html
+ 
+<a href="http://travis-ci.org/stevieb9/devel-examine-subs"><img src="https://secure.travis-ci.org/stevieb9/devel-examine-subs.png"/></a> <a href="https://coveralls.io/r/stevieb9/devel-examine-subs"><img src="https://coveralls.io/repos/stevieb9/devel-examine-subs.png?branch=master"/></a>
+ 
+=end html
+
 L<https://github.com/stevieb9/devel-examine-subs>
 
+=head1 BUILD REPORTS
+
+Travis-CI: L<https://travis-ci.org/stevieb9/devel-examine-subs>
+
+CPAN Testers: L<http://matrix.cpantesters.org/?dist=Devel-Examine-Subs>
 
 =head1 DEBUGGING
 
 If C<Devel::Trace::Subs> is installed, you can configure stack tracing.
 
-In your calling script, set C<$ENV{DTS_ENABLE} = 1> and 
-C<$ENV{DES_TRACE} = 1>.
+In your calling script, set C<$ENV{DES_TRACE} = 1>.
 
 See C<perldoc Devel::Trace::Subs> for information on how to access the traces.
 
@@ -1800,9 +1867,9 @@ See C<perldoc Devel::Trace::Subs> for information on how to access the traces.
 
 Information related to the 'pre_proc' phase core modules.
 
-=item C<perldoc Devel::Examine::Subs::Prefilter>
+=item C<perldoc Devel::Examine::Subs::Postprocessor>
 
-Information related to the 'pre_filter' phase core modules.
+Information related to the 'post_proc' phase core modules.
 
 =item C<perldoc Devel::Examine::Subs::Engine>
 
