@@ -1,9 +1,9 @@
 package Devel::Examine::Subs::Engine;
-use 5.012;
+use 5.008;
 use strict;
 use warnings;
 
-our $VERSION = '1.44_02';
+our $VERSION = '1.57_01';
 
 use Carp;
 use Data::Dumper;
@@ -20,8 +20,7 @@ BEGIN {
         import Devel::Trace::Subs qw(trace);
     };
 
-    if ($@){
-
+    if ($@ && ! exists $Devel::Examine::Subs{'trace'}){
         # override DTS's trace() function if necessary
         *trace = sub {};
     }
@@ -155,7 +154,7 @@ sub missing {
 
         for my $file (keys %$struct){
             for my $sub (keys %{$struct->{$file}{subs}}){
-                my @code = @{$struct->{$file}{subs}{$sub}{contents}};
+                my @code = @{$struct->{$file}{subs}{$sub}{code}};
 
                 my @clean;
 
@@ -187,7 +186,7 @@ sub lines {
         for my $file (keys %$struct){
             for my $sub (keys %{$struct->{$file}{subs}}){
                 my $line_num = $struct->{$file}{subs}{$sub}{start};
-                my @code = @{$struct->{$file}{subs}{$sub}{contents}};
+                my @code = @{$struct->{$file}{subs}{$sub}{code}};
                 for my $line (@code){
                     $line_num++;
                     push @{$return{$sub}}, {$line_num => $line};
@@ -201,7 +200,7 @@ sub objects {
     
     trace() if $ENV{TRACE};
 
-    # uses 'subs' pre_filter
+    # uses 'subs' post_processor 
 
     return sub {
         
@@ -382,7 +381,7 @@ sub inject_after {
 
             my $sub = $subs_hash->{$_};
 
-            my $num_injects = $p->{injects} // 1;
+            my $num_injects = defined $p->{injects} ? $p->{injects} : 1;
 
             push @processed, $sub->name;
 
@@ -482,7 +481,7 @@ Returns an aref.
 
 =head2 C<has>
 
-Takes C<$struct> from the output of the 'file_lines_contain' Pre-filter.
+Takes C<$struct> from the output of the 'file_lines_contain' Postprocessor.
 
 Returns an aref.
 
