@@ -1,9 +1,9 @@
 package Devel::Examine::Subs::Preprocessor;
-use 5.012;
+use 5.008;
 use strict;
 use warnings;
 
-our $VERSION = '1.44_02';
+our $VERSION = '1.57_01';
 
 use Carp;
 use Data::Dumper;
@@ -17,7 +17,7 @@ BEGIN {
         import Devel::Trace::Subs qw(trace);
     };
 
-    if ($@){
+    if ($@ && ! exists $Devel::Examine::Subs{'trace'}){
         *trace = sub {};
     }
 };
@@ -35,7 +35,6 @@ sub new {
 
     return $self;
 }
-
 sub _dt {
 
     trace() if $ENV{TRACE};
@@ -52,7 +51,6 @@ sub _dt {
 
     return $dt;
 }
-
 sub exists {
 
     trace() if $ENV{TRACE};
@@ -67,7 +65,6 @@ sub exists {
         return 0;
     }
 }
-
 sub module {
 
     trace() if $ENV{TRACE};
@@ -106,7 +103,6 @@ sub module {
         return \@subs;
     };
 }
-
 sub inject {
 
     trace() if $ENV{TRACE};
@@ -119,11 +115,30 @@ sub inject {
 
         my @file_contents = @{ $p->{file_contents} };
 
-        if ($p->{inject_use}) {
+        # after line number
+
+        my $rw = File::Edit::Portable->new;
+
+        if (defined $p->{line_num}){
+           
+            # inject after line number
+
+            $rw->splice(
+                file => $p->{file},
+                line => $p->{line_num},
+                insert => $p->{code},
+                copy => $p->{copy},
+            );
+        }
+        elsif ($p->{inject_use}){  
+            
+            # inject a use statement
 
             my $use = qr/use\s+\w+/;
+            
+            my $index;
 
-            my ($index) = grep {
+            ($index) = grep {
                 $file_contents[$_] =~ $use
             } 0..$#file_contents;
 
@@ -131,19 +146,21 @@ sub inject {
                 ($index) = grep {
                     $file_contents[$_] =~ /^package\s+\w+/
                 } 0..$#file_contents;
+                $index++;
             }
 
             if ($index) {
-                for (@{$p->{inject_use}}) {
-                    splice @file_contents, $index, 0, $_;
-                }
+                $rw->splice(
+                    file => $p->{file},
+                    line => $index,
+                    insert => $p->{inject_use},
+                    copy => $p->{copy},
+                );
             }
-
-            $p->{write_file_contents} = \@file_contents;
-
         }
+        elsif ($p->{inject_after_sub_def}){
 
-        if ($p->{inject_after_sub_def}) {
+            # inject code after sub definition
 
             my $code = $p->{inject_after_sub_def};
 
@@ -159,7 +176,11 @@ sub inject {
 
             my $is_multi = 0;
 
-            while (my ($i, $e) = each @file_contents){
+            my $i = -1;
+
+            for my $e (@file_contents){
+
+                $i++;
 
                 if ($e =~ /^\n/){
                     push @new_file, "\n";
@@ -215,7 +236,7 @@ sub replace {
 
         my $p = shift;
         my $exec = $p->{exec};
-        my $limit = $p->{limit} // -1;
+        my $limit = defined $p->{limit} ? $p->{limit} : -1;
 
         my @file = @{ $p->{file_contents} };
 
@@ -322,10 +343,9 @@ The data is returned early as mentioned in the L<DESCRIPTION>.
 =head2 C<inject>
 
 Parameters: C<{ inject_use =E<gt> ['use statement1;', 'use statement2;'] }>
-or C<{ inject_after_sub_def =E<gt> ['code line 1;', 'code line 2;'] }>
+or C<{ inject_after_sub_def =E<gt> ['code line 1;', 'code line 2;'] }> or C<{ line_num =E<gt> $num, code => \@code }>
 
-Injects each element of the array ref as either a use statement, or in the
-latter case, lines of code after a sub definition.
+Injects each element of the array ref as either a use statement, lines of code after a sub definition, or a block of code immediately after the line number.
 
 =head2 C<remove>
 
