@@ -1,9 +1,9 @@
 package Devel::Examine::Subs::Sub;
-use 5.012;
+use 5.008;
 use strict;
 use warnings;
 
-our $VERSION = '1.44_02';
+our $VERSION = '1.57_01';
 
 use Data::Dumper;
 
@@ -16,7 +16,7 @@ BEGIN {
         import Devel::Trace::Subs qw(trace);
     };
 
-    if ($@){
+    if ($@ && ! exists $Devel::Examine::Subs{'trace'}){
         *trace = sub {};
     }
 };
@@ -90,7 +90,7 @@ sub code {
 
     my $self = shift;
 
-    my $code = $self->{data}{contents};
+    my $code = $self->{data}{code};
 
     return $code;
 }
